<?php

namespace Drupal\sel\EventSubscriber;

use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * A subscriber for Sel configuration changes.
 */
class SelConfigChangeSubscriber implements EventSubscriberInterface {

  /**
   * The cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypePluginManager;

  /**
   * Constructs a SelConfigChangeSubscriber object.
   *
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cache_tags_invalidator
   *   The cache tags invalidator.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_plugin_manager
   *   The field type plugin manager.
   */
  public function __construct(
    CacheTagsInvalidatorInterface $cache_tags_invalidator,
    FieldTypePluginManagerInterface $field_type_plugin_manager,
  ) {
    $this->cacheTagsInvalidator = $cache_tags_invalidator;
    $this->fieldTypePluginManager = $field_type_plugin_manager;
  }

  /**
   * Empties field info cache on sel config change events.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The event.
   */
  public function onChange(ConfigCrudEvent $event) {
    if ($event->getConfig()->getName() === 'sel.settings') {
      $this->cacheTagsInvalidator->invalidateTags(['entity_field_info']);
      $this->fieldTypePluginManager->clearCachedDefinitions();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ConfigEvents::SAVE][] = ['onChange'];
    $events[ConfigEvents::DELETE][] = ['onChange'];

    return $events;
  }

}
